@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 Yubico AB - See COPYING
+ * Copyright (C) 2014-2018 Yubico AB - See COPYING
  */
 
 #include <u2f-server.h>
@@ -15,11 +15,12 @@
 #include <string.h>
 #include <getopt.h>
 #include <unistd.h>
+#include <sys/types.h>
+#include <pwd.h>
 
 #include "cmdline.h"
 
-int main(int argc, char *argv[])
-{
+int main(int argc, char *argv[]) {
   int exit_code = EXIT_FAILURE;
   struct gengetopt_args_info args_info;
   char buf[BUFSIZE];
@@ -32,20 +33,19 @@ int main(int argc, char *argv[])
   char *origin = NULL;
   char *appid = NULL;
   char *user = NULL;
+  struct passwd *passwd;
   const char *kh = NULL;
   const char *pk = NULL;
   u2fh_devs *devs = NULL;
   unsigned i;
   unsigned max_index = 0;
 
-
   if (cmdline_parser(argc, argv, &args_info) != 0)
     exit(EXIT_FAILURE);
 
   if (args_info.help_given) {
     cmdline_parser_print_help();
-    printf
-        ("\nReport bugs at <https://github.com/Yubico/libu2f-server>.\n");
+    printf("\nReport bugs at <https://github.com/Yubico/pam-u2f>.\n");
     exit(EXIT_SUCCESS);
   }
 
@@ -58,45 +58,44 @@ int main(int argc, char *argv[])
 
   s_rc = u2fs_init(&ctx);
   if (s_rc != U2FS_OK) {
-    fprintf(stderr, "error: u2fs_init (%d): %s\n", s_rc,
-            u2fs_strerror(s_rc));
+    fprintf(stderr, "error: u2fs_init (%d): %s\n", s_rc, u2fs_strerror(s_rc));
     exit(EXIT_FAILURE);
   }
 
   if (args_info.origin_given)
     origin = args_info.origin_arg;
   else {
-    if (!strcpy(buf, PAM_PREFIX))
+    if (!strcpy(buf, PAM_PREFIX)) {
       fprintf(stderr, "strcpy failed\n");
-    if (gethostname(buf + strlen(PAM_PREFIX), BUFSIZE - strlen(PAM_PREFIX))
-        == -1) {
+      exit(EXIT_FAILURE);
+    }
+    if (gethostname(buf + strlen(PAM_PREFIX), BUFSIZE - strlen(PAM_PREFIX)) ==
+        -1) {
       perror("gethostname");
       exit(EXIT_FAILURE);
     }
     origin = buf;
   }
 
+  if (args_info.verbose_given)
+    fprintf(stderr, "Setting origin to %s\n", origin);
+
   s_rc = u2fs_set_origin(ctx, origin);
   if (s_rc != U2FS_OK) {
-    printf("error: u2fs_set_origin (%d): %s\n", s_rc, u2fs_strerror(s_rc));
+    fprintf(stderr, "error: u2fs_set_origin (%d): %s\n", s_rc,
+            u2fs_strerror(s_rc));
     exit(EXIT_FAILURE);
   }
 
   if (args_info.appid_given)
     appid = args_info.appid_arg;
   else {
-    if (args_info.origin_given) {
-      if (!strcpy(buf, PAM_PREFIX))
-        fprintf(stderr, "strcpy failed\n");
-      if (gethostname
-          (buf + strlen(PAM_PREFIX), BUFSIZE - strlen(PAM_PREFIX)) == -1) {
-        perror("gethostname");
-        exit(EXIT_FAILURE);
-      }
-    }
-    appid = buf;
+    appid = origin;
   }
 
+  if (args_info.verbose_given)
+    fprintf(stderr, "Setting appid to %s\n", appid);
+
   s_rc = u2fs_set_appid(ctx, appid);
   if (s_rc != U2FS_OK) {
     fprintf(stderr, "error: u2fs_set_appid (%d): %s\n", s_rc,
@@ -107,15 +106,16 @@ int main(int argc, char *argv[])
   if (args_info.username_given)
     user = args_info.username_arg;
   else {
-    user = getlogin();
-    if (!user) {
-      perror("getlogin");
+    passwd = getpwuid(getuid());
+    if (passwd == NULL) {
+      perror("getpwuid");
       exit(EXIT_FAILURE);
     }
+    user = passwd->pw_name;
   }
 
-  if (u2fh_global_init(args_info.debug_flag ? U2FS_DEBUG : 0) != U2FH_OK
-      || u2fh_devs_init(&devs) != U2FH_OK) {
+  if (u2fh_global_init(args_info.debug_flag ? U2FH_DEBUG : 0) != U2FH_OK ||
+      u2fh_devs_init(&devs) != U2FH_OK) {
     fprintf(stderr, "Unable to initialize libu2f-host\n");
     exit(EXIT_FAILURE);
   }
@@ -129,8 +129,8 @@ int main(int argc, char *argv[])
 
   if (h_rc == U2FH_NO_U2F_DEVICE) {
     for (i = 0; i < TIMEOUT; i += FREQUENCY) {
-      fprintf(stderr,
-              "\rNo U2F device available, please insert one now, you have %2d seconds",
+      fprintf(stderr, "\rNo U2F device available, please insert one now, you "
+                      "have %2d seconds",
               TIMEOUT - i);
       fflush(stderr);
       sleep(FREQUENCY);
@@ -150,8 +150,8 @@ int main(int argc, char *argv[])
   }
 
   if (h_rc != U2FH_OK) {
-    fprintf(stderr,
-            "\rNo device found. Aborting.                                         \n");
+    fprintf(stderr, "\rNo device found. Aborting.                              "
+                    "           \n");
     exit(EXIT_FAILURE);
   }
 
@@ -162,10 +162,8 @@ int main(int argc, char *argv[])
     exit(EXIT_FAILURE);
   }
 
-  h_rc =
-      u2fh_register(devs, p, origin, &response,
-                    U2FH_REQUEST_USER_PRESENCE);
-  if (h_rc != U2FS_OK) {
+  h_rc = u2fh_register(devs, p, origin, &response, U2FH_REQUEST_USER_PRESENCE);
+  if (h_rc != U2FH_OK) {
     fprintf(stderr, "Unable to generate registration challenge, %s (%d)\n",
             u2fh_strerror(h_rc), h_rc);
     exit(EXIT_FAILURE);
